# Preact LineChart

## Usage
```javascript
import { default as graph, defaultOptions } from 'line-chart';
graph({ el: document.querySelect('someSelector'), data: kewlData, ....})
```

## Options
```javascript
export const defaultOptions = {
    // top, bottom, left and right, numbers
    margin,
    // the data object, should be a array of data
    data: [],
    // The y axis text format 
	  yAxisFormat: (iter, value) => value,
    // the x axis text format
    xAxisFormat: (iter, value) => {
		if (iter % 2 === 0 ) {
			return value;
		}
    },
    // the rendering into the header of the mouse
    mouseHeaderRender: (data) => 'data_header',
    // rendering into the body of the mouse hover
    mouseBodyRender: (data) => 'data_body',
    // if there is no data it uses a 10 day span for empty data sets
    defaultXDomain: [],
    // a bit more complex, if there are no or miniscule changes in the Y Axis it will change the scale according to this value
    defaultYDomain: [],
    // trumps all others, if set these will be the domains
    forceYDomain: [],
    forceXDomain: [],
    // domain callback functions
    yDomain: ([min, max], default, force) => {
      return [0, max * 2]
    },
    xDomain: ([minDate, maxDate], default, force) => {
      return [new Date(1985), maxDate]
    },
    // Remove the grid lines in the chart 
    removeGrid: false
    // domain ticks
    xTicks: 10,
    yTicks: 5,
    // d3 will round nicely
    useXNice: true,
    useYNice: true,
    // Set the chart component in development mode
    dev: false,
};
```

## Setting domains with callback

```
const yCallBack = ([min, max]) => {
  // do something amazing
  // return an array with min max
  return [newMin, newMax]
}

const xCallBack = ([min, max]) => {
  // do something amazing
  // return an array with min max
  return [newMin, newMax]
}

```